library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.util.all;

package simeck_pkg is

  -------------------------------------------
  --  block size, key size,  word size; 
  -------------------------------------------
  constant word_sz       : natural := 24;
  constant block_num     : natural := 2;  -- 2 * word_sz is the block size
  constant oblock_num    : natural := 8; --outer block size
  constant block_sz      : natural := block_num * word_sz;    -- 2 *16 
  constant oblock_sz     : natural := oblock_num * word_sz;
  
  constant rounds        : natural := 6;
  constant steps         : natural := 18;
  
  constant reg_rounds    : natural := 3;
  constant reg_steps     : natural := 5;
  
  ----------------------------------------------
  
    constant key_num       : natural := 4;  -- 4 * word_sz is the key size
	

  
  
  
  --------------------------------------------
  -- word_elem;
  --------------------------------------------
  subtype  word_elem               is std_logic_vector (word_sz -1 downto 0);
  type     word_elem_vector        is array (natural range <>) of word_elem; 
  --------------------------------------------
  type     message_elem            is array (block_num -1 downto 0) of word_elem;
  type	   block_elem              is array (oblock_num -1 downto 0) of word_elem;

  --------------------------------------------
   
  constant cont_round       : std_logic_vector(63 downto 0) := x"ffffffffffffffff";  
   
       type     key_elem                is array (key_num -1   downto 0) of word_elem;
  --------------------------
  subtype mode_ty is std_logic_vector(1 downto 0);
  constant load_mode   : mode_ty := "10";
  constant run_mode    : mode_ty := "01";

  --------------------------------------------
  --functions
  --------------------------------------------
 
  function shift_message(a:word_elem; r: message_elem)  return message_elem;
  function shift_key(a:word_elem; r: key_elem) return key_elem;


  function shift_message_serial(a:std_logic; r: message_elem)  return message_elem;
  function shift_key_serial(a:std_logic; r: key_elem) return key_elem;
  --function shift_block(a:word_elem; r: block_elem) return block_elem; 
  
  function is_load (m : mode_ty) return std_logic;
  function is_run (m : mode_ty) return std_logic;	
  function inc2(m: mode_ty) return mode_ty;
   
  function permut_block(a1:message_elem;a2:message_elem;RC1_in:std_logic_vector(5 downto 0);RC2_in:std_logic_vector(5 downto 0);  r: block_elem)  return block_elem;
   
  
end package;

------------------------------------------------------------------------
-- package body
------------------------------------------------------------------------

package body simeck_pkg is

  -----------------------------------------------------------------------
  
  function shift_message(a:word_elem; r: message_elem)  return message_elem
  is
    variable z: message_elem;
    constant i_min : integer := 0;
    constant i_max : integer := 1;
  begin
    
    z(i_max) :=  a;
    
    for i in i_max - 1 downto i_min loop 
      z(i) := r(i + 1);
    end loop;
    
    return z;	  
  end function;
 
  function shift_block(a1:word_elem;a2:word_elem; r: block_elem)  return block_elem
  is
    variable z: block_elem;
    constant i_min : integer := 0;
    constant i_max : integer := 7;
  begin

    z(7) := a2;
    z(6) := a1;
    z(5) := r(7);
    z(4) := r(6);
    z(3) := r(5);
    z(2) := r(4);
    z(1) := r(3);
    z(0) := r(2);

    return z;
  end function;
------------------------------------------
function permut_block(a1:message_elem;a2:message_elem;RC1_in:std_logic_vector(5 downto 0);RC2_in:std_logic_vector(5 downto 0);  r: block_elem)  return block_elem
  is
    variable z: block_elem;
    constant i_min : integer := 0;
    constant i_max : integer := 7;
  begin
  
  z(7) := r(1);
  z(6) := r(0);

z(5)(word_sz-1 downto 6) := not (r(7)(word_sz-1 downto 6) xor a1(1)(word_sz-1 downto 6));
z(5)(5 downto 0) := r(7)(5 downto 0) xor a1(1)(5 downto 0) xor RC1_in;

z(4) := not(r(6) xor a1(0));

z(3) := r(5);
z(2) := r(4);

z(1)(word_sz-1 downto 6) := not (r(3)(word_sz-1 downto 6) xor a2(1)(word_sz-1 downto 6));
z(1)(5 downto 0) := r(3)(5 downto 0) xor a2(1)(5 downto 0) xor RC2_in;



z(0) := not(r(2) xor a2(0));


    return z;
  end function;



  function shift_message_serial(a:std_logic; r: message_elem)  return message_elem
  is
    variable z: message_elem;
    constant i_min : integer := message_elem'low;
    constant i_max : integer := message_elem'high;
  begin
    
    z(i_max)(word_sz - 1) :=  a;
    z(i_min)(word_sz - 1) := r(i_max)(0);
    
    for i in word_sz - 2 downto 0 loop 
      z(i_max)(i) := r(i_max)(i + 1);
      z(i_min)(i) := r(i_min)(i + 1);
    end loop;
    
    return z;	  
  end function;
  
  
  
  function shift_key(a:word_elem; r: key_elem) return key_elem
  is
    variable z: key_elem;
    constant i_min : integer := key_elem'low;
    constant i_max : integer := key_elem'high;
    
  begin
    
    z(i_max) := a;
    
    for i in i_max - 1 downto i_min loop 
      z(i) := r(i + 1);
    end loop;
    
    return z;		
  end function;
  
  
  function shift_key_serial(a:std_logic; r: key_elem) return key_elem
  is
    variable z: key_elem;
    
  begin
    
    z(3)(word_sz - 1) := a;
    z(2)(word_sz - 1) := r(3)(0);
    z(1)(word_sz - 1) := r(2)(0);
    z(0)(word_sz - 1) := r(1)(0);
    
    for i in word_sz - 2 downto 0 loop 
      z(3)(i) := r(3)(i + 1);
      z(2)(i) := r(2)(i + 1);
      z(1)(i) := r(1)(i + 1);
      z(0)(i) := r(0)(i + 1);
    end loop;

    return z;		
  end function;
  
  function is_load (m: mode_ty) return std_logic
  is
  begin
    return m(1);		
  end function;
  
  function is_run (m: mode_ty) return std_logic 
  is
  begin
    return m(0);		
  end function;
  
   
  
  function inc2(m: mode_ty) return mode_ty
  is
    variable r: mode_ty;
  begin
 --   r := m ror 1;
      r := r(0) & r(1);  
  return r;
  end function;	  
  
  
end package body;  
